# Copyright (C) 2022 Bjarne von Horn (vh at igh dot de).
#
# This file is part of the PdCom library.
#
# The PdCom library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# The PdCom library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
# License for more .
#
# You should have received a copy of the GNU Lesser General Public License
# along with the PdCom library. If not, see <http://www.gnu.org/licenses/>.

from pdcom5._PdComWrapper import Process, Subscriber, Transmission, Subscription
import socket
import ssl


class MySub(Subscriber):
    def __init__(self, mode, proc, path):
        super().__init__(mode)
        self._sub1 = Subscription(self, proc, path)

    def stateChanged(self, s: Subscription):
        print("State changed:", s.state)

    def newValues(self, time):
        print("New Values:", self._sub1.value)


class MyProc(Process):
    def __init__(self, host: str, port: int, ca_file_name: str):
        super().__init__()
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
        ctx.load_verify_locations(cafile=ca_file_name)
        ctx.verify_mode = ssl.CERT_REQUIRED
        ctx.check_hostname = True
        self._tlssock = ctx.wrap_socket(self._sock, server_hostname=host)
        self._tlssock.connect((host, port))
        self._running = True
        self._connected = False

    def write(self, b):
        self._tlssock.sendall(b)

    def read(self, b):
        return self._tlssock.recv_into(b)

    def connected(self):
        print("Yaay, connected!")
        self._connected = True

    def listReply(self, vars, dirs):
        print("Vars:")
        for var in vars:
            print("  ", var.path)
        print("Dirs:")
        for dir in dirs:
            print("  ", dir)


if __name__ == "__main__":
    from sys import argv

    if len(argv) < 3:
        print("Usage:", argv[0], "<hostname> <ca file path>")
        exit(-1)
    p = MyProc(argv[1], 4523, argv[2])
    while not p._connected:
        p.asyncData()
    sub = MySub(Transmission.event_mode, p, "/input/klemme1")
    p.list()
    try:
        while p._running:
            p.asyncData()
    except KeyboardInterrupt:
        # need to unsubscribe before Process goes away
        sub._sub1.cancel()
        pass
